; TODO
; * Copy level data to RAM. This will allow to:
;   - pack level data in ROM (two tiles per byte)
;   - animate tiles
;   - easier (=faster) addressing

	processor 6502
	include vcs.h

	org $F000

PAL_COLORS = 0
PAL_60hz = 0

BALLCOLOR = $4a
BGCOLOR = $94

BALL_HEIGHT = 4

BALLANIMDELAY_STANDARD = 4
BALLANIMFRAMES = 15	;(0..)
BALLMOVEDELAY_STANDARD = 8

PS_temp = $80
Temp = $81
PtrLo = $80	; reuse
PtrHi = $81	; reuse

ptr0 = $82	; 2-bytes pointers to sprites data
ptr1 = $84
ptr2 = $86
ptr3 = $88
ptr4 = $8a
ptr5 = $8c

LoopCount = $8e
ScanCount = $8f
ScanNumber = $90
BallTabPtr = $91
RowsNumber = $92
BallX = $93
BallY = $94
BallPos = $95
BallMoveDelay = $96
BallAnimDelay = $97
BallAnimPtr = $98

LevelDataPtr = $99
LevelDataFlipFlop = $9a

LevelRam = $9b
; Free from $e3 (or $e4?)
MapX = $e4	; Player X and Y positions and pointer on the map
MapY = $e5
MapPtr = $e6

TilesLeft = $e7
LivesLeft = $e8
LevelRowsNumber = $e9

; ----- MACROS -----
; Thomas Jentzsch's SLEEP macro
 MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      bit $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM
; ------------------

; --------- Game entry point is here ----------
Start
	SEI  ; Disable interrupts, if there are any.
	CLD  ; Clear BCD math bit.
	LDX  #$FF
	TXS  ; Set stack to beginning ($FF).

	LDA #0
B1      STA 0,X
	DEX
	BNE B1	; Clear RAM and 0-page registers

	JSR InitGame
MainLoop
	JSR  VerticalBlank ;Execute the vertical blank.
	JSR  CheckSwitches ;Check console switches.
	JSR  GameCalc      ;Do calculations during Vblank
	JSR  DrawScreen    ;Draw the screen
	JSR  OverScan      ;Do more calculations during overscan
	JMP  MainLoop      ;Continue forever.

; ----------

VerticalBlank	subroutine		;*********************** VERTICAL BLANK HANDLER
	LDX  #0
	LDA  #2
;	STA  WSYNC  
;	STA  WSYNC	; Extra?
	STA  WSYNC
	STA  VSYNC ;Begin vertical sync.
	STA  WSYNC ; First line of VSYNC
	STA  WSYNC ; Second line of VSYNC.

	IF PAL_60hz
	LDA  #52	; 37 vblank lines (45 if PAL) with timer + 3 lines with WSYNCs
	ELSE
	LDA  #44	; 37 vblank lines (45 if PAL) with timer + 3 lines with WSYNCs
	ENDIF
	STA  TIM64T

	LDA #0
	STA CXCLR

	STA  WSYNC ; Third line of VSYNC.
	STA  VSYNC ; (0) End VSYNC
	RTS  

CheckSwitches	subroutine	 ;*************************** CONSOLE SWITCH HANDLER
	LDA #0
	STA COLUBK  ; Background will be black.
;	LDA SWCHA       ;Read joystick 0
;	STA Joystick0   ;Store for later use

;	LDA SWCHB
;	AND #%00000001
;	BNE NoResetGame
	;JSR GameInit	; Se RESET e' premuto, reinizializza il gioco
	;JSR NewLevel
.NoResetGame
	RTS

OverScan	subroutine   ;***************************** OVERSCAN CALCULATIONS
; -- First, do vblank and clear some stuff
	LDA #2
	STA WSYNC  ;Finish scanline 192.
	STA VBLANK ; Make TIA output invisible,
	;
	LDY #0
	STY PF0
	STY PF1
	STY PF1
	STY GRP0
	STY GRP1
	STY ENAM0
	STY ENAM1
	STY ENABL

	IF PAL_60hz
	LDX #36		; PAL
	ELSE
	LDX #30		; NTSC
	ENDIF

.KillLines
	STA WSYNC
	DEX
	BNE .KillLines
	RTS

; Input: X: sprite's X position
CalcSprXPos	subroutine
	TXA	; absolute X pixel
	lsr
	lsr
	lsr
	lsr
	tay	; /16
	
	sta PS_temp
	
	;lda spr0x
	TXA
	and #15
	clc
	adc PS_temp
	cmp #15
	
	tya
	adc #0
	tay
	sty PS_temp
	iny
	iny
	iny
	
	;lda spr0x
	TXA
	and #15
	clc
	adc PS_temp	; calculate the remainder for the /15
	; Use remainder for fine adjustment
	eor #15
	sbc #7	; -8 to +7
	asl
	asl
	asl
	asl

;	RETURN:
;	A: Fine Movement (offset)
;	Y: Position
	RTS

CopyLevelToRam	subroutine	; Unpack and copy level data from ROM to RAM
	LDA #1	; Current level
	ASL
	TAX
	DEX
	LDA LevelPointers,X
	STA PtrLo
	DEX
	LDA LevelPointers,X
	STA PtrHi

	LDX #71			; Point to the last byte of Level Ram
	LDA #0
.ClearLevelRam			; Clear level Ram
	STA LevelRam,X
	DEX
	BPL .ClearLevelRam

	LDY #5
	LDA (PtrLo),Y
	STA BallY
	DEY
	LDA (PtrLo),Y
	STA BallX
	DEY
	LDA (PtrLo),Y
	STA MapPtr		; Starting position
	DEY
	LDA (PtrLo),Y
	STA TilesLeft		; Tiles to destroy
	DEY
	LDA (PtrLo),Y		; Level Color
;	STA 
	DEY
	LDA (PtrLo),Y		; Load number of bytes for this level
	TAY

	CMP #41			; Small level?
	BEQ .SmallLevel
	LDX #71			; Big Level! Point to the last byte of Level Ram
;	LDA #12
;	STA LevelRowsNumber
	JMP .BigLevel
.SmallLevel
;	LDX #35			; Small level: point halfway throu Level Ram
	LDX #53
	LDA BallY
	CLC
	SBC #19			; Ball Y movement has a resolution of 2 pixels...!
	STA BallY
;	LDA #71
;	LDA #6
;	STA LevelRowsNumber
.BigLevel

.CopyLoop
	LDA (PtrLo),Y
	STA LevelRam,X
	DEX
	DEY
	CPY #5
	BNE .CopyLoop

	RTS

GameCalc	subroutine ;******************************* GAME CALCULATION ROUTINES
	; Ball animation delay
	LDY BallAnimPtr
	LDA BallAnimTab,Y
	STA BallTabPtr		; Set animation pointer to animation data tables
	DEC BallAnimDelay
	BNE .NoZeroBallAnimDelay
	LDA #BALLANIMDELAY_STANDARD
	STA BallAnimDelay	; Reset ball animation delay and go to the next frame of animation
	DEC BallAnimPtr		; Next frame of animation
	BPL .NoZeroBallAnimPtr	; No more frames of animation?
	LDA #BALLANIMFRAMES	; Make animation wrap!
	STA BallAnimPtr
.NoZeroBallAnimPtr
.NoZeroBallAnimDelay	; No change of animation frame

;	LDA #3
;	STA BallTabPtr	; Reset pointer to animation tables

; -- Ball Movement (temporary, just for demo)
;	DEC BallX
	DEC BallMoveDelay
	BNE .NoMoveBall
	LDA BallX
	CMP #103
	BCS .NoIncBallX
	INC BallX
	LDA #BALLMOVEDELAY_STANDARD
	STA BallMoveDelay
	JMP .NoMoveBall
.NoIncBallX
;	JMP .NoMoveBall
.NoMoveBallY
	LDA BallY
	CMP #137
	BCC .NoMoveBall
	DEC BallY
	LDA #BALLMOVEDELAY_STANDARD
	STA BallMoveDelay
.NoMoveBall

; --

; -- Prepare stuff for the next frame...
	LDA #191
	STA ScanCount

;	LDA LevelRowsNumber
	LDA #12
	STA RowsNumber

	;LDA #38
;	LDA #71
;	STA LevelDataPtr

	LDA #2
	STA LevelDataFlipFlop
; --
	RTS

InitGame	subroutine
	LDA #1
	STA ScanNumber

	LDA  #5		; Playfield reflected and qith priority over sprites
;	LDA  #53		; Playfield reflected and qith priority over sprites
	STA  CTRLPF

	; X limits:
	; (55)63 - 103(111)
	;LDA #55
	;STA BallX
	;LDA #191
	;STA BallY

	LDA #3
	STA BallTabPtr

	LDA #BALLANIMDELAY_STANDARD
	STA BallAnimDelay

	LDA #BALLMOVEDELAY_STANDARD
	STA BallMoveDelay

	LDA #BALLANIMFRAMES
	STA BallAnimPtr

;	LDA #71
;	STA LevelDataPtr

	JSR CopyLevelToRam
	;LDA #8
	;STA LevelRam+71

;	JSR InitScreenSprites

	RTS

DrawScreen	subroutine
	LDA INTIM
	BNE DrawScreen ; Whew!
	STA WSYNC
	STA VBLANK  ;End the VBLANK period with a zero.

;	LDA #3
;	LDX ScanNumber
;	BEQ NoIncBallTab
;	CLC
;	ADC #4
;NoIncBallTab
;	STA BallTabPtr

	LDA #71
	STA LevelDataPtr

	STA HMCLR
	JSR PositionBall
	STA WSYNC
	STA HMOVE
	JSR Position6Sprites
	STA HMCLR		; Very important...!
	STA WSYNC
	STA HMOVE
	JSR InitScreenSprites
	STA WSYNC
	;LDA #$1f
	LDA #BGCOLOR
	STA COLUP0
	STA COLUP1
	LDA #BALLCOLOR
	STA COLUPF

; Draw title
	LDA #%00000001
	STA PF1
	LDA #$ff
	STA PF2
	LDX #9
.LoopTitle
	STA WSYNC
	DEX
	BPL .LoopTitle
	LDA #0
	STA PF2
	

	LDA #7
	JSR Display6Sprite

;	LDA #%00000000
;	STA PF1
;	STA PF2

	DEC ScanNumber
	BPL .NoResetScanNumber
	INC ScanNumber
	INC ScanNumber
.NoResetScanNumber

; Draw footer
	STA WSYNC
	LDA #$ff
	STA PF2
	LDX #9
.LoopFooter
	STA WSYNC
	DEX
	BPL .LoopFooter
	LDA #0
	STA PF2
	STA PF1

;	LDA #191
;	LDA #172
	LDA #39
;	LDA #2
	STA ScanCount

.FinishScreen
	STA WSYNC
	DEC ScanCount
	BNE .FinishScreen

	RTS


InitScreenSprites	subroutine
;	JSR Position6Sprites

; Load pointers to sprites data
; --- Load pointers with next row of level data
;	LDX #0
;	LDY LevelDataPtr	; 3
;.LoopLoadBg
;	LDA LevelData,Y	; 4
;	STA ptr0,X	; 3
;	DEY	; 2
;	INX
;	INX
;	CPX #12
;	BNE .LoopLoadBg
;	STY LevelDataPtr

	LDX LevelDataPtr	; 3
;	LDX #71		; 2
; Unrolling loop to save some cycles...
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr0	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr1	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr2	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr3	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr4	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr5	; 3
	DEX	; 2
	STX LevelDataPtr	; 3
; ---

; Make a loop of the following stuff to save up rom space
;	LDA #<BgTiles0
;	STA ptr0
	LDA #>BgTiles0
	STA ptr0+1
;	LDA #<BgTiles1
;	STA ptr1
	LDA #>BgTiles1
	STA ptr1+1
;	LDA #<BgTiles2
;	STA ptr2
	LDA #>BgTiles2
	STA ptr2+1
;	LDA #<BgTiles3
;	STA ptr3
	LDA #>BgTiles3
	STA ptr3+1
;	LDA #<BgTiles4
;	STA ptr4
	LDA #>BgTiles4
	STA ptr4+1
;	LDA #<BgTiles5
;	STA ptr5
	LDA #>BgTiles5
	STA ptr5+1	; 60 cycles
	RTS

Position6Sprites	subroutine
	LDA     #$01
	STA     VDELP0
	STA     VDELP1
	LDA     #$03
	STA     NUSIZ0
	STA     NUSIZ1
	LDA     #$46
	STA     COLUP0
	STA     COLUP1

	LDA #0
	STA COLUBK
	STA COLUPF
	STA GRP0
	STA GRP1
	STA REFP0
	STA REFP1
	STA ENAM0
	STA ENAM1
	STA ENABL
;	STA RESP0
;	STA RESP1
	STA HMCLR

	STA     WSYNC

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP	; 20 cycles
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP	; 40 cycles total

;	LDY #6		; 2
;.DelayLoop_introscreensprites1
;	DEY		; 2 * 7
;	BPL .DelayLoop_introscreensprites1	; 3 * 6 + 2 =  cycles
;	NOP
;	NOP
;	SLEEP 40
	SLEEP 38

	STA RESP0	; Position sprites on the screen
	STA WSYNC

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	bit $00
;	NOP	; 19 cycles
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP	; 41 cycles

;	LDY #7		; 2
;.DelayLoop_introscreensprites2
;	DEY		; 2 * 7
;	BPL .DelayLoop_introscreensprites2	; 3 * 6 + 2 =  cycles

;	SLEEP 41
	SLEEP 39

	STA     RESP1

;	LDA     #$50    ; .. and shift them
;	LDA     HMP1
;	LDA     #$40
;	STA     HMP0

	;LDA     #$90    ; .. and shift them
	;LDA     HMP1
	LDA #%11110000
	STA HMP1
;	LDA     #$50
	LDA #%01000000
	STA     HMP0

	STA     WSYNC
	STA     HMOVE
	STA     WSYNC
	RTS

	org $fb00

PositionBall	subroutine
	LDX BallX
	JSR CalcSprXPos
	STA HMBL
	TYA
	TAX

	;LDX BallPos
	STA WSYNC
.BallXposLoop1
	DEX
	BPL .BallXposLoop1
	STA RESBL	; Enable (or disable) missile 0 graphics
	RTS

Display6Sprite
	STA LoopCount	; 3	(40 cycles)
	LDA #0
	STA HMP0
	STA HMP1
	LDA #BGCOLOR	; Sprites color
X1:
	STA     WSYNC

	; -7
	; 1st loop: 64 cycles to position at the end of the sprites and start from there
;	LDY #10		; 2
;DelayLoop1
;	DEY		; 2 * 11
;	BPL DelayLoop1	; 3 * 10 + 2 =  cycles
	; 56

	SLEEP 54

;	BIT $00	; = 64
;	JMP X3
	; Conditioned branch to make kernel start alternatively with the sprites and with the ball lines.
	LDX ScanNumber	; 3
	BNE BgLine	; 2
	JMP NormalLine	; 3	8	= 64

BgLine
	NOP		; 2+1	8	= 64
X3:
;	NOP		; 2
;	NOP		; 2
;	NOP		; 2
	STA COLUP0
	STA COLUP1
	LDY     LoopCount	; 3
	LDA     (ptr0),Y	; 5
	STA     GRP0		; 3	17
	LDA     (ptr1),Y	; 5
	STA     GRP1		; 3
	LDA     (ptr2),Y	; 5
	STA     GRP0		; 3	33
	LDA     (ptr5),Y	; 5
	STA     Temp		; 3
	LDA     (ptr4),Y	; 5
	TAX			; 2	48
	LDA     (ptr3),Y	; 5
	LDY     Temp		; 3	56
	STA     GRP1		; 3
	STX     GRP0		; 3
	STY     GRP1		; 3
	STA     GRP0		; 3	68

;	NOP	; 2
;	BIT $00	; 3
;	BIT $00	; 3	76	; End of sprites line
	DEC LoopCount		; 5
	BPL NormalLine		; 3 if branch is taken	76
;	JMP EndScreen
	JMP NextRow		; 78

NormalLine
; Beginning of non-sprites line (another 76 cycles)
;	STA HMOVE	; 3
;	LDA #70	; 2
;	STA HMCLR	; 3
;	STA HMBL	; 3

;	STA COLUBK	; 3

	LDX #0			; 2
	STX COLUP0		; 3
	STX COLUP1		; 3	8
	STX ENABL	; 3
;	NOP
;	STX HMOVE	; HMOVE must be hit around here...!
;	STA COLUBK
;	NOP

;	STA HMOVE	; HMOVE must be hit around here...!

;	STA COLUBK
;	DEC BallTabPtr	; 5

	
	;LDA #53		; 2
	;STA CTRLPF	; 3	Ball is 4 clocks
;	LDA #$ff	; 2
;	STA HMOVE
;	STA ENABL	; 3
;	STA HMOVE	; HMOVE must be hit around here...!


;	DEC BallTabPtr	; 5

; 45 cycles
;	LDX #2		; 2
	DEC ScanCount	; 5
	STX HMOVE	; HMOVE must be hit around here...!
	LDA ScanCount	; 3
	SEC		; 2
	SBC BallY	; 3
	ADC #BALL_HEIGHT; 2
	BCS DrawBall1	; 2/3
;	LDA #0		; 2
;	STA HMBL	; 3
	STX HMBL	; 3
	SLEEP 24
	JMP SkipDrawBall1	; 3	45
DrawBall1
	LDY BallTabPtr	; 3
	LDA BallSizeTab,Y	; 4
	STA CTRLPF	; 3
	LDA BallEnableTab,Y	; 4
	STA ENABL	; 3
;	STX ENABL	; 3	45
	LDA BallShiftTab,Y	; 4
	STA HMBL	; 3
	DEC BallTabPtr	; 5	45
SkipDrawBall1

; 38 cycles:
;EndLineLogic
;	LDY #3		; 2		; (5 cycles per) Y + 4
;DelayLoop2
;	DEY		; 2 * 4
;	BPL DelayLoop2	; 3 * 3 + 2 =  cycles
;	BIT $00
;	BIT $00
;	BIT $00
;	BIT $00
;	BIT $00
;	NOP

;	SLEEP 3

;	LDA #5		; 2
;	STA CTRLPF	; 3 Ball 4 clocks
;	LDA #$00
;	STA ENABL
;	STA HMOVE
	
;	STA HMCLR
	LDY LoopCount	; 3
	LDA TilesColor,Y	; 4
;	LDA #BGCOLOR	; 2

	DEC     LoopCount	; 5
	; Should hmove ball or change size on odd or even lines only depending on which line we've started with.
	; (so that it results that we always change its values on the same line).
	; For this purpose, must use a set of 2 precalculated tables with HMOVEs and CTRLPFs with alternated
	; values.
;	STA HMBL
;	LDX LoopCount	; 3
	BPL     X3		; 3	; 76
	;BIT $00
	JMP NextRow	; 78
;	JMP EndScreen	; 78

	org $fc00

; We have 2 blank lines here for a total of 150 cycles
NextRow		; 76 cycles -2 from the jump to here
;	STA HMCLR
	DEC RowsNumber	; 5
	BEQ EndScreen	; 2
;	BNE ContinueRow	; 3
;	JMP EndScreen
;ContinueRow
	NOP
	STA HMOVE	; 3
	LDA #0		; 2
	STA COLUP0	; 3
	STA COLUP1	; 3
;	STA HMOVE	; 3
;	LDA #7		; 2
;	STA LoopCount	; 3	; 17
;	STA HMOVE

;	STA ENABL

;	LDY BallTabPtr	; 3
;	CPY #0		; 2
;	BNE DrawBall	; 2/3
;	STA ENABL	; 3
;	SLEEP 7
;	JMP NoDrawBall	; 3	20
;DrawBall
;	LDA BallShiftTab,Y	; 4
;	STA HMBL	; 3
;	DEC BallTabPtr	; 5	20
;NoDrawBall

; --- Move Ball ---
;	LDX #2		; 2
	DEC ScanCount	; 5
	LDA ScanCount	; 3
	SEC		; 2
	SBC BallY	; 3
	ADC #BALL_HEIGHT; 2
	BCS DrawBall2	; 2/3
	LDA #0		; 2
;	STA HMBL	; 3
	STA ENABL	; 3
	SLEEP 22
	JMP SkipDrawBall2	; 3	47
DrawBall2
	LDY BallTabPtr	; 3
	LDA BallSizeTab,Y	; 4
	STA CTRLPF	; 3
	LDA BallEnableTab,Y	; 4
	STA ENABL	; 3
;	STX ENABL	; 3	45
	LDA BallShiftTab,Y	; 4
	STA HMBL	; 3
;	SLEEP 3
	DEC BallTabPtr	; 5	; 47
SkipDrawBall2

; --- Load pointers with next row of level data (60 cycles)
	LDX LevelDataPtr	; 3
;	LDX #71		; 2
; Unrolling loop to save some cycles...
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr0	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr1	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr2	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr3	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr4	; 3
	DEX	; 2
;	LDA LevelData,Y	; 4
	LDA LevelRam,X	; 4
	STA ptr5	; 3
	DEX	; 2
	STX LevelDataPtr	; 3
; ---


;	SLEEP 35
;	SLEEP 76
;	SLEEP 67
	SLEEP 5

;	STA HMCLR

	LDA #7		; 2
	STA LoopCount	; 3

;	LDY #0
;	LDA BallTabPtr		; 3
;	CMP #0
;	BEQ HideBallLine	; 2/3
;	BIT $00			; 3	12
;	JMP NoHideBallLine	; 3
;HideBallLine
;	NOP			; 2
;	STY ENABL		; 3	12
;NoHideBallLine
	; Load pointers to tiles here

	LDA #BGCOLOR	; 2
	LDX ScanNumber	; 3
	BNE BgLine2	; 2
	BIT $00		; 3
	JMP NormalLine	; 3	13
BgLine2
	NOP		; 2+1
	JMP X3		; 3	13
; 36
EndScreen
	LDA #0
	STA COLUP0
	STA COLUP1
	STA ENABL
	LDA #$6f
;	STA COLUBK
	STA WSYNC
	LDA     #0
	STA     GRP0
	STA     GRP1
	STA     GRP0
	STA     GRP1
;	STA ENAM0
;	STA ENAM1
;	STA ENABL
	RTS

	org $fd00

BallShiftTab
;	.byte 224, 0, 32, 208
;	.byte 0, 0, 0, 16	; very small
;	.byte 0, 0, 0, 16	; small
;	.byte 0, 240, 0, 16	; medium
;	.byte 0, 224, 0, 32	; ok big

	.byte 0, 240, 240, 240	; very small
	.byte 0, 240, 0, 240	; small
	.byte 0, 240, 0, 16	; medium
	.byte 0, 224, 0, 32	; big

; 224
; 0
; 32
; <- 2
; 0
; -> 2

;	.byte 0, 0, 0, 70
;	.byte 4, 0, 3, 0, 2, 0, 1, 0

BallSizeTab
	.byte 5, 5, 5, 5	; very small
	.byte 5, 21, 21, 5	; small
	.byte 21, 37, 37, 21	; medium
	.byte 37, 53, 53, 37	; big

BallEnableTab
	.byte 0, 0, 2, 0	; very small
	.byte 0, 2, 2, 0	; small
	.byte 2, 2, 2, 2	; medium
	.byte 2, 2, 2, 2	; big

BallAnimTab
	.byte 7, 11, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 11, 7, 3
;	.byte 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15
;	.byte 15, 15, 15, 15, 15, 11, 11, 11, 11, 11

; 5  = 1 clock
; 21 = 2 clocks
; 37 = 4 clocks
; 53 = 8 clocks

;	.byte 53, 37, 21, 5
;	.byte 5, 21, 37, 53
;	.byte 53, 0, 37, 0, 21, 0, 5, 0, 0

; Level length in bytes: 6*height + 1
; Levels are upside-down
LevelData
LevelData1
	; Level 1
	.byte 12 *6+5, $94, 16, 0 + 6*0, 0*6+63, 191-0*8
	; Level length, color, tiles to destroy, starting location offset (always treated as a 72-bytes map) as X+6*Y, Ball X, Ball Y
;	.byte $11,$11,$11,$11,$11,$11
;	.byte $11,$11,$11,$11,$11,$11
;	.byte $11,$11,$11,$11,$11,$11
;	.byte $11,$11,$11,$11,$11,$11
;	.byte $11,$11,$11,$11,$11,$11
;	.byte $11,$11,$11,$11,$11,$11	; 38

	.byte $18,$08,$28,$28,$08,$20
	.byte $08,$08,$08,$08,$08,$08
	.byte $08,$00,$10,$10,$00,$08
	.byte $10,$08,$08,$08,$08,$10
	.byte $38,$00,$08,$08,$00,$38
	.byte $00,$00,$08,$08,$00,$00
	.byte $38,$00,$08,$08,$00,$38
	.byte $10,$08,$08,$08,$08,$10
	.byte $08,$08,$00,$00,$08,$08
	.byte $08,$08,$08,$08,$08,$08
	.byte $08,$08,$08,$08,$08,$08
	.byte $18,$08,$30,$30,$08,$20
	
	.byte $18,$08,$08,$08,$08,$20
	.byte $08,$10,$10,$10,$10,$08
	.byte $08,$10,$00,$00,$10,$08
	.byte $08,$10,$00,$00,$10,$08
	.byte $08,$10,$10,$10,$10,$08
	.byte $18,$08,$08,$08,$08,$20	; 38
	.byte $18,$08,$08,$08,$08,$20
	.byte $08,$10,$10,$10,$10,$08
	.byte $08,$10,$00,$00,$10,$08
	.byte $08,$10,$00,$00,$10,$08
	.byte $08,$10,$10,$10,$10,$08
	.byte $18,$08,$08,$08,$08,$20	; 38

TilesColor
	IF PAL_COLORS
	.byte $00, $d4, $d6, $d8, $d8, $d8, $d6, $d4	; PAL
	ELSE
	.byte $00, $84, $86, $88, $88, $88, $86, $84	; NTSC
	ENDIF

; 00 - Empty
; 08 - Normal
; 10 - Can't destroy
; 18 - Arrow Right	->
; 20 - Arrow Left	<-
; 28 - Arrow Down	v
; 30 - Arrow Up		^
; 38 - Double Jump

LevelPointers
	.byte >LevelData1, <LevelData1

	org $fe00

BgTiles
BgTiles0
;BgTiles1
;BgTiles2
;BgTiles3
;BgTiles4
;BgTiles5
	; 0 Empty (Kills you)
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
BgTiles1
	; 1 Normal
	.byte %11111110
	.byte %11010110
	.byte %10101010
	.byte %11010110
	.byte %10101010
	.byte %11010110
	.byte %10101010
	.byte %11111110
BgTiles2
	; 2 Can't destroy
	.byte %11111110
	.byte %10111010
	.byte %11111110
	.byte %11111110
	.byte %11111110
	.byte %11111110
	.byte %10111010
	.byte %11111110
BgTiles3
	; 3 Arrow Right
	.byte %11111110
	.byte %11111110
	.byte %11110110
	.byte %10000010
	.byte %10000010
	.byte %11110110
	.byte %11111110
	.byte %11111110
BgTiles4
	; 4 Arrow Left
	.byte %11111110
	.byte %11111110
	.byte %11011110
	.byte %10000010
	.byte %10000010
	.byte %11011110
	.byte %11111110
	.byte %11111110
BgTiles5
	; 5 Arrow Down
	.byte %11111110
	.byte %11101110
	.byte %11000110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11111110

	; 6 Arrow Up
	.byte %11111110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11000110
	.byte %11101110
	.byte %11111110

	; 7 Double Jump
	.byte %11111110
	.byte %11000110
	.byte %10111010
	.byte %10101010
	.byte %10101010
	.byte %10111010
	.byte %11000110
	.byte %11111110

Number0
	.byte %01111100
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %01111100
Number1
	.byte %11111100
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %11110000
	.byte %01110000
	.byte %00110000
Number2
	.byte %11111110
	.byte %11000000
	.byte %01100000
	.byte %00110000
	.byte %00011000
	.byte %00001100
	.byte %11000110
	.byte %01111100
Number3
	.byte %01111100
	.byte %11000110
	.byte %00000110
	.byte %00000110
	.byte %00011100
	.byte %00000110
	.byte %11000110
	.byte %01111100
Number4
	.byte %00000110
	.byte %11111110
	.byte %11000110
	.byte %01100110
	.byte %01100110
	.byte %00110000
	.byte %00110000
	.byte %00011000
Number5
	.byte %01111100
	.byte %11000110
	.byte %00000110
	.byte %00000110
	.byte %11111100
	.byte %11000000
	.byte %11000000
	.byte %11111110

	org $FFFC
	.word Start
	.word Start
